#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <cctype>

#include <idlc.h>
#include <util.h>

std::ostream &operator << (std::ostream &ostr, Indent &ind)
{
	for (int i = 0; i < ind.indent_level; i++)
		ostr << '\t';
	
	for (int i = 0; i < ind.align_spaces; i++)
		ostr << ' ';
			
	return ostr;
}

static bool check_empty(const char *dirname)
{
	struct dirent ent, *entp;

	DIR *dir = opendir(dirname);
	if (!dir) {
		fprintf(stderr, "Cannot open directory \"%s\".\n",
		        dirname);
		
		throw UserError();
	}

	do {
		// readdir_r is buggy on osx 10.2, and will fail if errno is
		// non-zero.
		errno = 0;

		if (readdir_r(dir, &ent, &entp) && errno != EEXIST) {
			fprintf(stderr, "2 Cannot readdir on \"%s\": %s.\n",
			        dirname, strerror(errno));
		
			closedir(dir);
			throw UserError();
		}
	} while (entp && (!strcmp(ent.d_name, ".") ||
	                  !strcmp(ent.d_name, "..")));
		
	closedir(dir);
	
	if (!entp)
		return true;
	
	return false;
}

void makepath(const char *_path, bool require_empty)
{
	string path(_path);
	string::size_type pos = 0, last = 0;
	
	while (pos != string::npos) {
		pos = path.find('/', pos + 1);
		string component(path, 0, pos);
		
		if (component.length() - last == 1) {
			last = pos;
			continue;
		}

		last = pos;
		
		if (mkdir(component.c_str(), 0777) && errno != EEXIST) {
			fprintf(stderr, "Cannot create directory \"%s\": %s.\n",
			        component.c_str(), strerror(errno));

			throw UserError();
		}
	}
	
	if (require_empty && !check_empty(_path)) {
		fprintf(stderr, "Directory \"%s\" is not empty.\n", _path);
		throw UserError();
	}
}

static inline int hex_to_bin(int hex)
{
	if (hex >= 'a')
		return hex - 'a' + 10;

	if (hex >= 'A')
		return hex - 'A' + 10;

	return hex - '0';
}

void parse_guid(const char *guid_str, char *guid_bin)
{
	bool second_hex_digit = false;
	int i;

	for (i = 0; i < 36; i++) {
		if (i == 8 || i == 13 || i == 18 || i == 23) {
			if (guid_str[i] != '-')
				goto bad;
		} else {
			if (!isxdigit(guid_str[i]))
				goto bad;
			
			if (second_hex_digit) {
				*guid_bin++ |= hex_to_bin(guid_str[i]);
				second_hex_digit = false;
			} else {
				*guid_bin = hex_to_bin(guid_str[i]) << 4;
				second_hex_digit = true;
			}
		}
	}
	
	if (guid_str[i] == 0)
		return;

bad:
	yyerrorf("\"%s\" is not a valid GUID.", guid_str);
	throw UserError();
}
