#ifndef _ARCH_THREAD_H
#define _ARCH_THREAD_H

#include <kern/types.h>

namespace Threads {
	class Thread;
}

namespace Mem {
	class ProcAddrSpace;
}

namespace Arch {
	struct ArchThread {
		void *esp, *ebp;
		u8 jump_to_init;
		
		enum {
			size = 4096
		};
		
		void init(void *entry, void *arg1, void *arg2);
	};

	void switch_thread(Threads::Thread *dest, Threads::Thread *src);
	void set_aspace(Mem::ProcAddrSpace *aspace);

	namespace Priv {
		struct TSS {
			u16 prev_task;
			u16 reserved1;
			u32 esp0;
			u16 ss0;
			u16 reserved2;
			u32 esp1;
			u16 ss1;
			u16 reserved3;
			u32 esp2;
			u16 ss2;
			u16 reserved4;
			u32 cr3, eip, eflags;
			u32 eax, ecx, edx, ebx, esp, ebp, esi, edi;
			u16 es;
			u16 reserved5;
			u16 cs;
			u16 reserved6;
			u16 ss;
			u16 reserved7;
			u16 ds;
			u16 reserved8;
			u16 fs;
			u16 reserved9;
			u16 gs;
			u16 reserved10;
			u16 ldt;
			u16 reserved11;
			u16 trap;
			u16 iomap_base;
		};
		
		extern TSS tss;
	}
};

#endif
