// tests/threads.cc
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors or contributors be held liable for any damages
// arising from the use of this software.
// 
// Permission is hereby granted to everyone, free of charge, to use, copy,
// modify, prepare derivative works of, publish, distribute, perform,
// sublicense, and/or sell copies of the Software, provided that the above
// copyright notice and disclaimer of warranty be included in all copies or
// substantial portions of this software.

#include <kern/types.h>
#include <kern/libc.h>
#include <kern/time.h>
#include <kern/thread.h>

void wake(Time::KTimerEntry *entry)
{
	Threads::Blocker *b = static_cast<Threads::Blocker *>(entry->data);
	b->wake();
}

void clearflag(Time::KTimerEntry *entry)
{
	int *flag = static_cast<int *>(entry->data);
	*flag = 0;
}

void threadb(void *arg)
{
	Time::KTimerEntry timer(Time::monotonic_timers);
	Threads::ThreadBlocker tb(curthread);
	
	for (;;) {
		volatile int flag = 1;
		Time::Time now;
		Time::monotonic_clock.get_time(&now);
		
		now.seconds += 2;
		
		timer.func = clearflag;
		timer.data = (void *)(&flag);
		timer.arm(now);
		
		while (flag)
			printf("B");

		now.seconds += 2;

		tb.blocked = true;
		timer.func = wake;
		timer.data = &tb;
		timer.arm(now);
		
		curthread->block(&tb);
	}
}

extern System::Mem::Mappable Mem::physmem;

void threada(void *arg)
{
	Time::KTimerEntry timer(Time::monotonic_timers);
	Threads::ThreadBlocker tb(curthread);
	
	Threads::sched.new_thread(threadb, NULL, "thread b")->wake();

	for (;;) {
		volatile int flag = 1;
		Time::Time now;
		Time::monotonic_clock.get_time(&now);
		
		now.seconds++;
		
		timer.func = clearflag;
		timer.data = (void *)(&flag);
		timer.arm(now);
		
		while (flag)
			printf("A");

		now.seconds++;

		tb.blocked = true;
		timer.func = wake;
		timer.data = &tb;
		timer.arm(now);
		
		curthread->block(&tb);
	}
}

void run_test()
{
	Threads::sched.new_thread(threada, NULL, "thread a")->wake();
}
